/****************************************************************************
*
* FILENAME
*   SERIAL.CPP
*
* DESCRIPTION
*   Essential external definitions for the class Serial.
*
* CONTENTS
*   Boolean        Serial::AllSent       ()
*   Boolean        Serial::AnyErrors     ()
*   void           Serial::Break         (unsigned)
*   Boolean        Serial::BreakDetect   ()
*   void           Serial::ClearErrors   ()
*   void           Serial::Disable       ()
*   void           Serial::Enable        (unsigned, unsigned)
*   void           Serial::EnableInt     ()
*   Boolean        Serial::FrameError    ()
*   void           Serial::HandleError   ()
*   void           Serial::HandleMSR     ()
*   void           Serial::HandleRDR     ()
*   void           Serial::HandleTHR     ()
*   Boolean        Serial::HandleInt     ()
*   unsigned char  Serial::In            ()
*   void           Serial::InstallInt    ()
*   void interrupt Serial::Int0B         (...)
*   void interrupt Serial::Int0C         (...)
*   void interrupt Serial::Int13         (...)
*   Boolean        Serial::NoCarrier     ()
*   void           Serial::Out           (unsigned char)
*   Boolean        Serial::OverflowError ()
*   Boolean        Serial::ParityError   ()
*   void           Serial::RemoveInt     ()
*   void           Serial::RestoreInt    ()
*                  Serial::Serial        (Serial::ComPort)
*                  Serial::Serial        (Serial::ComPort, unsigned, unsigned)
*                  Serial::~Serial       ()
*   void           Serial::SetComPort    (Serial::ComPort)
*   void           Serial::SetFlowType   (Serial::FlowType)
*   void           Serial::SetRxFlowOn   (Boolean)
*   void           Serial::SetTxFlowOn   (Boolean)
*
*  unsigned char        Serial::intsInstalled
*  void     interrupt (*Serial::OldInt0B)(...)
*  void     interrupt (*Serial::OldInt0C)(...)
*  void     interrupt (*Serial::OldInt13)(...)
*  unsigned char        Serial::oldIntMask
*  Boolean              Serial::portEnabled[]
*  Serial              *Serial::portObject[]
*  Boolean              Serial::wasServiced[]
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* GLOBAL HEADER FILES
*
*****************************************************************************/

#include "serial.h"  // class Serial
#include <_null.h>   // Definition of NULL
#include "ascii.h"   // ASCII constant definitions

/****************************************************************************
*
* STATICS
*
*****************************************************************************/

// Interrupt handlers not yet installed
unsigned char Serial::intsInstalled = 0;

// Pointers to original interrupt handlers
void interrupt (*Serial::OldInt0B)(...);
void interrupt (*Serial::OldInt0C)(...);
void interrupt (*Serial::OldInt13)(...);

// Original 8259 interrupt controller mask register
unsigned char Serial::oldIntMask = 0;

// Flag indicates if serial port is enabled
Boolean Serial::portEnabled[] = { False, False, False, False };

// Pointers to serial port objects, since interrupt handler is static
Serial  *Serial::portObject[] = { NULL, NULL, NULL, NULL };

// Flag indicates if serial port was cause of interrupt
Boolean Serial::wasServiced[] = { False, False, False, False };

/****************************************************************************
*
* FILENAME
*   SERIAL.CPP
*
* FUNCTION NAME
*   Serial::AllSent
*
* PURPOSE
*   Test if all data has been sent to the DCE.
*
* DESCRIPTION
*   A true return results ONLY when ALL transmit buffers are empty.
*
* EXTERNAL REFERENCES
*   enum  Boolean
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   June 19, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

Boolean            // RETURN: True  = All data transmitted
                   //         False = All data not transmitted

Serial::AllSent()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Get the CURRENT state of transmission
  GetLSR();

  // Return True only if ALL transmit buffers are empty
  return ( TxIsEmpty() && ( LSR & MASK_LSR_TSR )) ? True : False;
}

/****************************************************************************
*
* FILENAME
*   SERIAL.CPP
*
* FUNCTION NAME
*   Serial::AnyErrors
*
* PURPOSE
*   Test if any error in reception has occured.
*
* DESCRIPTION
*   All errors, including a break, are abnormal receiver events and are
*   reported by this routine.
*
*   See ClearError() for usage of error test routines.
*
* EXTERNAL REFERENCES
*   enum  Boolean
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

Boolean              // RETURN: True  = Error has occured
                     //         False = No errors have occured

Serial::AnyErrors()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Return True or False depending on the error history
  return ( errorState & MASK_LSR_ERRORS ) ? True : False;
}

/****************************************************************************
*
* FILENAME
*   SERIAL.CPP
*
* FUNCTION NAME
*   Serial::Break
*
* PURPOSE
*   Transmits a break signal.
*
* DESCRIPTION
*   A break signal is always valid, no matter what the line settings are.
*   This feature is often used to signify bit rate changes, imminent line
*   disconnection, etc. The exact meaning of a break is usage dependent.
*
*   A typical break signal is 250 milliseconds, but this is user selectable
*   by specifing the duration (1-65535 ms).
*
* EXTERNAL REFERENCES
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

void                 // RETURN: Nothing is returned

Serial::Break
(
  unsigned duration  // INPUT: Duration of break in milliseconds
)

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Save flow control state
  Boolean oldTxFlow = txFlowOn;

  // Stop transmitting data
  SetTxFlowOn( False );

  // Start sending a break
  MCR |= MASK_LCR_BREAK;
  SetMCR();

  // Wait for duration milliseconds
  delay( duration );

  // Stop sending a break
  MCR &= ~MASK_LCR_BREAK;
  SetMCR();

  // Restore flow control state
  SetTxFlowOn( oldTxFlow );
}

/****************************************************************************
*
* FILENAME
*   SERIAL.CPP
*
* FUNCTION NAME
*   Serial::BreakDetect
*
* PURPOSE
*   Test if a break signal was received.
*
* DESCRIPTION
*   A break signal is always valid, no matter what the line settings are.
*   This feature is often used to signify bit rate changes, imminent line
*   disconnection, etc. The exact meaning of a break is usage dependent.
*
*   See ClearError() for usage of error test routines.
*
* EXTERNAL REFERENCES
*   enum  Boolean
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

Boolean                 // RETURN: True  = Break received
                        //         False = Break not received

Serial::BreakDetect()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Return True or False depending on the error history
  return ( errorState & MASK_LSR_BREAK ) ? True : False;
}

/****************************************************************************
*
* FILENAME
*   SERIAL.CPP
*
* FUNCTION NAME
*  Serial::ClearErrors
*
* PURPOSE
*   Initialize the error history to the 'no error' state.
*
* DESCRIPTION
*   This routine is intended to be called BEFORE transfering data that will
*   be checked for errors. Since received data is always queued, errors
*   cannot be detected on a character by character basis. Instead, errors
*   can only be detected between calls to this routine.
*
*   All errors, including breaks, result in a loss of data that is NOT put
*   in the receive queue. For most types of communications, errors can be
*   ignored completely or are handled by higher level protocols. Called by
*   the constructor.
*
* EXTERNAL REFERENCES
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

void                   // RETURN: Nothing is returned

Serial::ClearErrors()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Reset error history to show no errors have occured yet
  errorState = 0;
}

/****************************************************************************
*
* FILENAME
*   SERIAL.CPP
*
* FUNCTION NAME
*   Serial::Disable
*
* PURPOSE
*   Disables a serial port.
*
* DESCRIPTION
*   This routine was designed to allow the queues to be resized by calling
*   Enable() afterwards. Called by the destructor.
*
* EXTERNAL REFERENCES
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

void               // RETURN: Nothing is returned

Serial::Disable()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Make sure the port is enabled
  if( portEnabled[ portNumber ] )
  {
    // Let the DCE know we're offline, and disable uart interrupts
    MCR = 0;
    SetMCR();

    // Disable uart interrupts
    IER = 0;
    SetIER();

    // Destroy the queues
    delete rxQueue;
    delete txQueue;

    // Disable interrupts, and remove the interrupt handlers if possible
    RemoveInt();
  }

  // Mark this serial port as invalid for the interrupt routine
  portEnabled[ portNumber ] = False;
  portObject[ portNumber ] = NULL;

  // Queues are invalid
  rxQueue = txQueue = NULL;
}

/****************************************************************************
*
* FILENAME
*   SERIAL.CPP
*
* FUNCTION NAME
*   Serial::Enable
*
* PURPOSE
*   Enable a serial port for use.
*
* DESCRIPTION
*   This routine was designed to allow the queues to be resized by first
*   calling Disable(). Queues from 256 to 65520 characters can be used.
*   Called by the constructor.
*
* EXTERNAL REFERENCES
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

Boolean              // RETURN: True  = Enable succeeded
                     //         False = Enabled failed

Serial::Enable
(
  unsigned rxQSize,   // INPUT: Size of receive queue (in characters)
  unsigned txQSize    // INPUT: Size of transmit queue (in characters)
)

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Initialize return value to failure
  Boolean retval = False;

  // Make sure the port is disabled
  if( !portEnabled[ portNumber ] )
  {

    // Create the queues
    rxQueue = new SerialQueue( rxQSize );
    txQueue = new SerialQueue( txQSize );

    if(rxQueue && txQueue)
    {
      // Install the interrupt handlers if needed, and enable interrupts
      InstallInt();

      // Enable receiver, DCE signal, and error interrupt sources
      IER = MASK_IER_RDR | MASK_IER_MSR | MASK_IER_ERROR;
      SetIER();

      // Let the DCE know we're online, and enable uart interrupts
      MCR = MASK_MCR_DTR | MASK_MCR_RTS | MASK_MCR_USER2;
      SetMCR();

      // Mark this serial port as valid for the interrupt routine
      portEnabled[ portNumber ] = True;

      // success
      retval = True;
    }
    else
    {
      // Delete both queues
      if(rxQueue) delete rxQueue;
      if(txQueue) delete txQueue;
      rxQueue = txQueue = NULL;
    }
  }

  return retval;
}

/****************************************************************************
*
* FILENAME
*   SERIAL.CPP
*
* FUNCTION NAME
*   Serial::EnableInt
*
* PURPOSE
*   Enable serial interrupts, preserving the original setting.
*
* DESCRIPTION
*   This routine simply clears the 8259 PIC mask bits for IRQ3 and IRQ4,
*   allowing them to be recognized by the processor.
*
* EXTERNAL REFERENCES
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

inline

void                 // RETURN: Nothing is returned

Serial::EnableInt()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Save the old 8259A PIC register
  oldIntMask = inportb( 0x21 );

  // Enable interrupts 3 & 4
  outportb( 0x21, oldIntMask & ~( 0x18 ) );
}

/*****************************************************************************
*
* FILENAME
*   SERIAL.CPP
*
* FUNCTION NAME
*   Serial::FrameError
*
* PURPOSE
*   Test if a frame error in reception has occured.
*
* DESCRIPTION
*   Frame errors indicate a missing stop bit. These errors are generally
*   caused by line noise.
*
*   See ClearError() for usage of error test routines.
*
* EXTERNAL REFERENCES
*   enum  Boolean
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

Boolean               // RETURN: True  = Frame error detected
                      //         False = Frame error not detected

Serial::FrameError()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Return True or Flase depending on the error history
  return ( errorState & MASK_LSR_FRAME ) ? True : False;
}

/****************************************************************************
*
* FILENAME
*   SERIAL.CPP
*
* FUNCTION NAME
*   Serial::HandleError
*
* PURPOSE
*   Interrupt routine to handle uart error conditions, or a break.
*
* DESCRIPTION
*   Data in error is not queued; this ensures the queue is always valid. If
*   character sequencing is critical, the occurance of an error can be
*   detected by AnyErrors(). See ClearError() for details on errors.
*
*   Note that this function is private and inline!
*
* EXTERNAL REFERENCES
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

inline

void                   // RETURN: Nothing is returned

Serial::HandleError()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Accumulate an error history
  errorState |= LSR;

  // Read the erroneous data and ignore it
  GetRDR();
}

/****************************************************************************
*
* FILENAME
*   SERIAL.CPP
*
* FUNCTION NAME
*   Serial::HandleMSR
*
* PURPOSE
*   Interrupt routine to handle DCE signal changes.
*
* DESCRIPTION
*   Hardware flow control is handled by this routine. Since any change in any
*   of the DCE signals will cause this routine to be called, the MSR data
*   member is always valid. This is an added bonus; it is much faster to
*   check a memory location, than it is to access a potentially slow bus
*   device.
*
* DESCRIPTION
*   Frame errors indicate a missing stop bit. These errors are generally
*   caused by line noise.
*
*   See ClearError() for usage of error test routines.
*
* EXTERNAL REFERENCES
*   enum  Boolean
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

inline

void                 // RETURN: Nothing is returned

Serial::HandleMSR()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // If hardware flow control is enabled, check for a CTS change
  if( flowType == FLOW_HARD )
  {
    SetTxFlowOn( ( MSR & MASK_MSR_CTS ) ? True : False );
  }
}

/****************************************************************************
*
* FILENAME
*   SERIAL.CPP
*
* FUNCTION NAME
*   Serial::HandleRDR
*
* PURPOSE
*   Interrupt routine to handle received data.
*
* DESCRIPTION
*   When flow control is enabled, it is exercised by this routine for the
*   receiver. If the DCE does not honor a request to stop transmitting
*   quickly, data will be lost but no OverflowError() will be reported.
*
*   Note that this function is private and inline!
*
* EXTERNAL REFERENCES
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

inline

void

Serial::HandleRDR()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Get the received data
  GetRDR();

  // Check for software flow control
  if( flowType == FLOW_SOFT )
  {
    //Check for XON
    if( RDR == ASCII_XON )
    {
      SetTxFlowOn( True );
    }
    // Check for XOFF
    else if( RDR == ASCII_XOFF )
    {
      SetTxFlowOn( False );
    }
    // Otherwise stuff it in the queue
    else
    {
      rxQueue->PutData( RDR );
    }
  }
  else
  {
    // Otherwise stuff it in the queue
    rxQueue->PutData( RDR );
  }

  // Assert flow control if necessary
  if( rxQueue->IsNearFull() && rxFlowOn )
  {
    SetRxFlowOn( False );
  }
}

/****************************************************************************
*
* FILENAME
*   SERIAL.CPP
*
* FUNCTION NAME
*   Serial::HandleTHR
*
* PURPOSE
*   Interrupt routine to handle transmitted data.
*
* DESCRIPTION
*   Transmmitter interrupts will be disabled if their is nothing to transmit,
*   or if the transmitter flow has been disabled by flow control.
*
*   Note that this function is private and inline!
*
* EXTERNAL REFERENCES
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

inline

void                 // RETURN: Nothing is returned

Serial::HandleTHR()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Receiver software flow control has priority over data
  if( flowChar )
  {
    // Transmit the flow character
    SetTHR( flowChar );
    flowChar = ASCII_NUL;
  }
  else
  {
    if( !txFlowOn || TxIsEmpty() )
    {
      // Turn off transmitter interrupts if nothing to transmit
      IER &= ~MASK_IER_THR;
      SetIER();
    }
    else
    {
      // Otherwise transmit the next character
      SetTHR( txQueue->GetData() );
    }
  }
}

/****************************************************************************
*
* FILENAME
*   SERIAL.CPP
*
* FUNCTION NAME
*   Serial::HandleInt
*
* PURPOSE
*   Main interrupt routine that decides what, if anything, needs handling.
*
* DESCRIPTION
*   Checks interrupt enable bits before handling interrupts, since the ID
*   register doesn't have a 'no interrupt' state. This prevents accidentally
*   handling someone else's interrupt.
*
* EXTERNAL REFERENCES
*   enum  Boolean
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

Boolean              // RETURN: True  = Interrupt was handled
                     //         False = Interrupt was not handled

Serial::HandleInt()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Initialize return value to no interrupt handled
  Boolean retval = False;

  // Get the current LSR into its shadow
  GetLSR();

  // Check for source interrupt
  if( ( LSR & MASK_LSR_ERRORS ) && ( IER & MASK_IER_ERROR ) )
  {
    // Real error interrupt occured
    HandleError();
    retval = True;
  }
  else if( ( LSR & MASK_LSR_RDR ) && ( IER & MASK_IER_RDR ) )
  {
    // Real receiver interrupt occured
    HandleRDR();
    retval = True;
  }
  else if( ( LSR & MASK_LSR_THR ) && ( IER & MASK_IER_THR ) )
  {
    // Real transmitter interrupt occured
    HandleTHR();
    retval = True;
  }
  else if( ( IER & MASK_IER_MSR ) && ( GetMSR(), ( MSR & MASK_MSR_CHANGE ) ) )
  {
    // Real DCE signal interrupt occured
    HandleMSR();
    retval = True;
  }
  // Return if interrupt handled/not handled
  return retval;
}

/*****************************************************************************
*
* FILENAME
*   SERIAL.CPP
*
* FUNCTION NAME
*   Serial::In
*
* PURPOSE
*   Input a character from an enabled serial port.
*
* DESCRIPTION
*   This routine does NOT check if the port is enabled, and can potentially
*   hang the controlling program in a busy-wait state. To avoid delays,
*   make sure the port is enabled, and test a call to IsRxEmpty().
*
* EXTERNAL REFERENCES
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/


unsigned char  // RETURN: Character from receive queue

Serial::In()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // If there is no data, negate flow contol if necessary
  if( !rxFlowOn && rxQueue->IsNearEmpty() )
  {
    SetRxFlowOn( True );
  }

  // Busy-wait until there is something in the queue
  while( RxIsEmpty() )
  {
  }

  // Return the received data
  return rxQueue->GetData();
}

/****************************************************************************
*
* FILENAME
*   SERIAL.CPP
*
* FUNCTION NAME
*   Serial::InstallInt
*
* PURPOSE
*   Install serial interrupt handlers, preserving the original vectors.
*
* DESCRIPTION
*   The interrupt handlers are installed for all four ports if they have not
*   already been installed.
*
* EXTERNAL REFERENCES
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

void                  // RETURN: Nothing is returned

Serial::InstallInt()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Only install when necessary
  if( !intsInstalled )
  {
    // Disable interrupts
    disable();

    // Install new IRQ3 (vector 0x0b) handler
    OldInt0B = getvect( 0x0b );
    setvect( 0x0b, Int0B );

    // Install new IRQ4 (vector 0x0c) handler
    OldInt0C = getvect( 0x0c );
    setvect( 0x0c, Int0C );

    // Install disk bios (vector 0x13) handler
    OldInt13 = getvect( 0x13 );
    setvect( 0x13, ( void interrupt ( * )( ... )) Int13 );

    // Both vectors are installed, so safe to enable IRQ3 and IRQ4 if needed
    EnableInt();

    // Allow interrupts again
    enable();
  }
  // Interrupts now installed
  intsInstalled++;
}

/****************************************************************************
*
* FILENAME
*   SERIAL.CPP
*
* FUNCTION NAME
*   Serial::Int0B
*
* PURPOSE
*   Interrupt routine for COM2 and COM4 (IRQ 3).
*
* DESCRIPTION
*   Identical to COM1 and COM3 routine, in all but the ports handled.
*
* EXTERNAL REFERENCES
*   enum  Boolean
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

void           // RETURN: Nothing is returned

interrupt      // This is an interrupt routine

Serial::Int0B
(
  ...          // INPUT: Interrupt stack frame (unused)
)

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Nothing has been done yet
  wasServiced[ COM2 ] = wasServiced[ COM4 ] = False;

  // If COM2 is enabled, then it MIGHT be the source of the interrupt
  if( portEnabled[ COM2 ] )
  {
    // Check for and handle any interrupt
    wasServiced[ COM2 ] = portObject[ COM2 ]->HandleInt();
  }

  // If COM4 is enabled, then it MIGHT be the source of the interrupt
  if( portEnabled[ COM4 ] )
  {
    // Check for and handle any interrupt
    wasServiced[ COM4 ] = portObject[ COM4 ]->HandleInt();
  }

  // If interrupt was not from an enabled port, it was from somewhere else
  if( !(wasServiced[COM2] || wasServiced[ COM4 ] ) )
  {
    // Call the original interrupt handler (its not our problem)
    OldInt0B();
  }
  else
  {
    // Otherwise it was a serial interrupt, so acknowledge it as handled.
    outportb( 0x20, 0x20 );
  }
}

/****************************************************************************
*
* FILENAME
*   SERIAL.CPP
*
* FUNCTION NAME
*   Serial::Int0C
*
* PURPOSE
*   Interrupt routine for COM1 and COM3 (IRQ 4).
*
* DESCRIPTION
*   Identical to COM2 and COM4 routine, in all but the ports handled.
*
* EXTERNAL REFERENCES
*   enum  Boolean
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

void            // RETURN: Nothing is returned

interrupt       // This is an interrupt routine

Serial::Int0C
(
  ...           // INPUT: Interrupt stack frame (unused)
)

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Nothing has been done yet
  wasServiced[ COM1 ] = wasServiced[ COM3 ] = False;

  // If COM1 is enabled, then it MIGHT be the source of the interrupt
  if( portEnabled[ COM1 ] )
  {
    // Check for and handle any interrupt
    wasServiced[ COM1 ] = portObject[ COM1 ]->HandleInt();
  }

  // If COM3 is enabled, then it MIGHT be the source of the interrupt
  if( portEnabled[ COM3 ] )
  {
    // Check for and handle any interrupt
    wasServiced[ COM3 ] = portObject[ COM3 ]->HandleInt();
  }

  // If interrupt was not from an enabled port, it was from somewhere else
  if( !( wasServiced[ COM1 ] || wasServiced[ COM3] ) )
  {
    // Call the original interrupt handler (it's not our problem)
    OldInt0C();
  }
  else
  {
    // Otherwise it was a serial interrupt, so acknowledge it as handled.
    outportb( 0x20, 0x20 );
  }
}

/****************************************************************************
*
* FILENAME
*   SERIAL.CPP
*
* FUNCTION NAME
*   Serial::Int13
*
* PURPOSE
*   Interrupt routine for disk access (INT 0x13) flow control.
*
* DESCRIPTION
*   This routine turns off the data flow to all serial ports whenever a disk
*   access is detected. The data flow is restored when the disk access is
*   completed.
*
*   THIS IS EXPERIMENTAL!
*
*   This function slows disk accesses, and may not detect accesses either.
*
* EXTERNAL REFERENCES
*   enum  Boolean
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

// Let the compiler know we modified the args indirectly
#pragma argsused

void              // RETURN: Nothing is returned

interrupt         // This is an interrupt routine

Serial::Int13
(
  unsigned bp,    // INPUT: Base pointer register
  unsigned di,    // INPUT: Destination index register
  unsigned si,    // INPUT: Source index register
  unsigned ds,    // INPUT: Data segment register
  unsigned es,    // INPUT: Extra segment register
  unsigned dx,    // INPUT: DX register
  unsigned cx,    // INPUT: CX register
  unsigned bx,    // INPUT: BX register
  unsigned ax,    // INPUT: AX rexister
  unsigned ip,    // INPUT: Instruction pointer register
  unsigned cs,    // INPUT: Code segment register
  unsigned flags  // INPUT: FLAGS register
)

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Previous flow control states
  Boolean oldRxFlow[ COMPORTS ];
  Boolean oldTxFlow[ COMPORTS ];

  Serial *p;
  int port;

  // Turn data flow off to all ports
  enable();
  for( port = COM1; port <= COM4; port++ )
  {
    if( portEnabled[ port ])
    {
      p = portObject[ port ];
      oldRxFlow[ port ] = p->rxFlowOn;
      oldTxFlow[ port ] = p->txFlowOn;
      p->SetRxFlowOn( False );
      p->SetTxFlowOn( False );
      while( p->flowChar )
      {
      }
    }
  }
  disable();

  // Call disk routine
  // Note: The original BP and DS are not passed, since they keep track of
  // our stack frame and data segment
  _AX = ax; _BX = bx; _CX = cx; _DX = dx;
  _ES = es; _SI = si; _DI = di;
  OldInt13();
  ax = _AX; bx = _BX; cx = _CX; dx = _DX;
  es = _ES; si = _SI; di = _DI;
  flags = _FLAGS;

  // Restore data flow to all ports
  for( port = COM1; port <= COM4; port++ )
  {
    if( portEnabled[ port ] )
    {
      p = portObject[ port ];
      p->SetRxFlowOn( oldRxFlow[ port ] );
      p->SetTxFlowOn( oldTxFlow[ port ] );
    }
  }
}

/****************************************************************************
*
* FILENAME
*   SERIAL.CPP
*
* FUNCTION NAME
*   Serial::NoCarrier
*
* PURPOSE
*   Get the status of the data carrier detect (DCD) signal.
*
* DESCRIPTION
*   This is a very fast test that does not acesss the physical port. It is
*   always correct, thanks to the HandleMSR() routine called by the interrupt
*   handler.
*
* EXTERNAL REFERENCES
*   enum  Boolean
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

Boolean               // RETURN: True  = DCD is asserted
                      //         False = DCD is negated

Serial::NoCarrier()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // MSR in memory ALWAYS reflects the port...and is faster to access
  return ( MSR & MASK_MSR_DCD ) ? False : True;
}

/*****************************************************************************
*
* FILENAME
*   SERIAL.CPP
*
* FUNCTION NAME
*   Serial::Out
*
* PURPOSE
*   Output a character to an enabled serial port.
*
* DESCRIPTION
*   This routine does NOT check if the port is enabled, and can potentially
*   hang the controlling program in a busy-wait state. To avoid delays,
*   make sure the port is enabled, and test a call to IsTxFull();
*
* EXTERNAL REFERENCES
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

void                  // RETURN: Nothing is returned

Serial::Out
(
  unsigned char data  // INPUT: Character to output to the serial port
)

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Busy wait until there is room in the queue
  while( TxIsFull() )
  {
  }

  // Interpret software flow control characters
  if( flowType == FLOW_SOFT )
  {
    SetRxFlowOn( ( data == ASCII_XON ) ? True : False );

    if( data != ASCII_XON && data != ASCII_XOFF )
    {
      txQueue->PutData( data );

      // Enable transmitter interrupts, if the DCE hasn't yet disabled them
      if( txFlowOn )
      {
        IER |= MASK_IER_THR;
        SetIER();
      }
    }
  }
  else
  {
    // Put the data in the transmit queue
    txQueue->PutData( data );

    // Enable transmitter interrupts, if the DCE hasn't yet disabled them
    if( txFlowOn )
    {
      IER |= MASK_IER_THR;
      SetIER();
    }
  }
}

/****************************************************************************
*
* FILENAME
*   SERIAL.CPP
*
* FUNCTION NAME
*   Serial::OverflowError
*
* PURPOSE
*   Test if an overflow error in reception has occured.
*
* DESCRIPTION
*   This error is serious; it means the interrupt handler did not react
*   quickly enough, and data was lost. The fault may not be with the
*   interrupt handler, since DMA and servicing other interupts can delay
*   servicing a port. Should you detect this error, please contact the
*   author.
*
*   See ClearError() for usage of error test routines.
*
* EXTERNAL REFERENCES
*   enum  Boolean
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

Boolean                   // RETURN: True  = Overflow detected
                          //         False = Overflow not detected

Serial::OverflowError()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Return True or Flase depending on the error history
  return ( errorState & MASK_LSR_OVERFLOW ) ? True : False;
}

/****************************************************************************
*
* FILENAME
*   SERIAL.CPP
*
* FUNCTION NAME
*   Serial::ParityError
*
* PURPOSE
*   Test if a parity error in reception has occured.
*
* DESCRIPTION
*   Parity errors are only valid when PARITY_ODD, or PARITY_EVEN is used.
*   These errors are generally caused by line noise. Parity is not a secure
*   error check method, since it is easily fooled.
*
*   See ClearError() for usage of error test routines.
*
* EXTERNAL REFERENCES
*   enum  Boolean
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

Boolean                   // RETURN: True  = Parity error detected
                          //         False = Parity error not detected

Serial::ParityError()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Return True or Flase depending on the error history
  return ( errorState & MASK_LSR_PARITY ) ? True : False;
}

/****************************************************************************
*
* FILENAME
*   SERIAL.CPP
*
* FUNCTION NAME
*   Serial::RemoveInt
*
* PURPOSE
*   Remove serial interrupt handlers, restoring the original vectors.
*
* DESCRIPTION
*   The vectors are only removed if they are installed, and none of the four
*   possible ports is enabled.
*
* EXTERNAL REFERENCES
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

void                 // RETURN: Nothing is returned

Serial::RemoveInt()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Can remove ONLY if ALL serial ports are disabled
  intsInstalled--;

  if( !intsInstalled )
  {
    // Disable all interrupts
    disable();

    // Restore IRQ3 and IRQ4 to previous state
    RestoreInt();

    // Restore previous interrupt vectores
    setvect( 0x0b, OldInt0B );
    setvect( 0x0c, OldInt0C );
    setvect( 0x13, OldInt13 );

    // Allow interrupts again
    enable();
  }
}

/****************************************************************************
*
* FILENAME
*   SERIAL.CPP
*
* FUNCTION NAME
*   Serial::RestoreInt
*
* PURPOSE
*
*   Restore serial interrupts to their original settings.
*
* DESCRIPTION
*
*   This routine simply resets the 8259 PIC mask bits for IRQ3 and IRQ4 to
*   their original settings.
*
* EXTERNAL REFERENCES
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

void                  // RETURN: Nothing is returned

Serial::RestoreInt()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Restore the 8259A PIC register to its original value
  outportb( 0x21, oldIntMask );
}

/****************************************************************************
*
* FILENAME
*   SERIAL.CPP
*
* FUNCTION NAME
*   Serial::Serial
*
* PURPOSE
*   Constructor for class Serial.
*
* DESCRIPTION
*   Any one of the four COM ports can be used, but a port cannot have more
*   than one Serial object associated with it. This limitation is logical,
*   and is a result of the static nature of the interrupt routines. All other
*   parameters can be adjusted by using the appropriate member functions.
*
*   This constructor does not enable the port.
*
* EXTERNAL REFERENCES
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

Serial::Serial
(
  Serial::ComPort comPort  // INPUT: Serial port to use
)

/****************************************************************************
*
* BASE CLASS AND CONSTANT INITIALIZATIONS
*
*****************************************************************************/

: Uart( comPort )

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Set the COM port
  SetComPort(comPort);
}

/****************************************************************************
*
* FILENAME
*   SERIAL.CPP
*
* FUNCTION NAME
*   Serial::Serial
*
* PURPOSE
*   Constructor for class Serial.
*
* DESCRIPTION
*   Any one of the four COM ports can be used, but a port cannot have more
*   than one Serial object associated with it. This limitation is logical,
*   and is a result of the static nature of the interrupt routines. All other
*   parameters can be adjusted by using the appropriate member functions.
*
*   This constructor enables the port.
*
* EXTERNAL REFERENCES
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

Serial::Serial
(
  Serial::ComPort comPort,   // INPUT: Serial port to use
  unsigned rxQSize,          // INPUT: Size of receive queue (characters)
  unsigned txQSize           // INPUT: Size of transmit queue (characters)
)

/****************************************************************************
*
* BASE CLASS AND CONSTANT INITIALIZATIONS
*
*****************************************************************************/

: Uart( comPort )

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Set the COM port
  SetComPort(comPort);

  // Enable the serial port with given size of queues
  Enable( rxQSize, txQSize );
}

/****************************************************************************
*
* FILENAME
*   SERIAL.CPP
*
* FUNCTION NAME
*   Serial::~Serial
*
* PURPOSE
*   Destructor for class Serial.
*
* DESCRIPTION
*   The port is disabled, which removes all queues and possibly removes the
*   interrupt handlers.
*
*   Once destroyed, the COM port is completely ignored.
*
* EXTERNAL REFERENCES
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

Serial::~Serial()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Disable the port (remove queues, and possibly interrupt handlers)
  Disable();
}

/****************************************************************************
*
* FILENAME
*   SERIAL.CPP
*
* FUNCTION NAME
*   Serial::SetComPort
*
* PURPOSE
*   Set the COM port used.
*
* DESCRIPTION
*   This function can be safely called at any time. The port must not be
*   used until a call to Enable is made!
*
* EXTERNAL REFERENCES
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

void                       // RETURN: Nothing is returned

Serial::SetComPort
(
  Serial::ComPort comPort  // INPUT: COM port to use
)

/****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

{
  // Disable port if enabled
  if( portEnabled[ comPort ] )
  {
    Disable();
  }

  // Set the COM port
  Uart::SetComPort( comPort );

  // Initialize static object pointer, for static interrupt handler
  portObject[ comPort ] = this;

  // Turn off flow control
  SetFlowType( FLOW_NONE );

  // Clear error history
  ClearErrors();
}

/****************************************************************************
*
* FILENAME
*   SERIAL.CPP
*
* FUNCTION NAME
*   Serial::SetFlowType
*
* PURPOSE
*   Set the flow control method.
*
* DESCRIPTION
*   Hardware (FLOW_HARD) using RTS/CTS signals, software (FLOW_SOFT) using
*   XON/XOFF characters, or none (FLOW_NONE) methods of flow control can be
*   selected. Calling this routine resets the present flow control state,
*   and therefore it is best to call after creating a serial port object
*   (always created with FLOW_NONE), or after disabling an active port.
*
* EXTERNAL REFERENCES
*   enum  Boolean
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

void                     // RETURN: Nothing is returned

Serial::SetFlowType
(
  Serial::FlowType flow
)

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Turn on the data flow
  rxFlowOn = txFlowOn = True;

  // No software flow control character to send
  flowChar = ASCII_NUL;

  // Set the flow control method
  flowType = flow;
}

/****************************************************************************
*
* FILENAME
*   SERIAL.CPP
*
* FUNCTION NAME
*   Serial::SetRxFlowOn
*
* PURPOSE
*   Send a flow control request to the DCE.
*
* DESCRIPTION
*   This routine is intended to be used to manually control the receiver's
*   flow control.
*
* EXTERNAL REFERENCES
*   enum  Boolean
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

void                 // RETURN: Nothing is returned

Serial::SetRxFlowOn
(
  Boolean state      // INPUT: True  = Flow control asserted
                     //        False = Flow control negated
)

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Check to see if there is a change
  if( state != rxFlowOn )
  {
    // Set new flow control state
    rxFlowOn = state;

    // Do the state change for software or hardware flow control
    if( flowType == FLOW_SOFT )
    {
      // Software flow control priority override of the transmitter
      flowChar = ( state ) ? ASCII_XON : ASCII_XOFF;

      // Enable the transmitter interrupts
      IER |= MASK_IER_THR;
      SetIER();
    }
    else if( flowType == FLOW_HARD )
    {
      // Let the DCE know our state
      SetRTS( state );
    }
  }
}

/****************************************************************************
*
* FILENAME
*   SERIAL.CPP
*
* FUNCTION NAME
*   Serial::SetTxFlowOn
*
* PURPOSE
*   Impose transmitter flow control by enabling/disabling the transmitter.
*
* DESCRIPTION
*   This routine both changes the transmitter flow setting, as well as
*   enable/disable the transmitter.
*
* EXTERNAL REFERENCES
*   enum  Boolean
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

void                 // RETURN: Nothing is returned

Serial::SetTxFlowOn
(
  Boolean state      // INPUT: True  = Flow control asserted
                     //        False = Flow control negated
)

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  if( state != txFlowOn )
  {
    // Set the transmitter flow control state
    txFlowOn = state;

    // Enable & disable transmitter interrupts
    IER = ( state ) ?
      ( TxIsEmpty() ? IER & ~MASK_IER_THR: IER | MASK_IER_THR  ):
      (flowChar     ? IER |  MASK_IER_THR: IER & ~MASK_IER_THR );

    SetIER();
  }
}

/****************************************************************************/
